package core

import (
	"chatgpt/util"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"strconv"
	"sync"
)

// wg
var wg sync.WaitGroup

// proxyAddr
// 代理配置
var proxyAddr string = util.GetProxyServer()

// apiKey
// GPT Api Key
var apiKey string = util.Conf.Get("gpt", "api_key")

// privateKey
// 签名密钥
var privateKey string = util.Conf.Get("init", "private_key")

// TimeoutValue
// 超时配置
var TimeoutValue string = util.Conf.Get("init", "timeout")

// Action
// 处理请求
//
//	@param w
//	@param r
func Handler(w http.ResponseWriter, r *http.Request) {
	w.Header().Add("Access-Control-Allow-Origin", "*")
	w.Header().Add("Access-Control-Allow-Methods", "POST, GET, PUT, DELETE, OPTIONS")
	w.Header().Add("Content-Type", "application/json")
	Timeout, _ := strconv.Atoi(TimeoutValue)
	if r.Method == "POST" {
		var requestData RequestData
		raw, _ := ioutil.ReadAll(r.Body)
		json.Unmarshal(raw, &requestData)

		if privateKey != "" && requestData.Sign != MakeSign(requestData, privateKey) {
			fmt.Fprintln(w, BuildResponse(privateKey, 0, "sign error", ResponseData{}))
			return
		}

		question := util.Base64{
			Content: []byte(requestData.Words),
		}.Decode()
		if len(question) > 0 {
			gpt := &GPT{
				ApiKey:  apiKey,
				Proxy:   proxyAddr,
				Timeout: Timeout,
			}
			wg.Add(1)
			go func() {
				defer wg.Done()
				if result, err := gpt.GetAnswer(question); err == nil {
					fmt.Fprintln(w, BuildResponse(privateKey, 1, "success", ResponseData{
						"answer": result.Choices,
					}))
				}
			}()
			wg.Wait()
			return
		}
	}
	fmt.Fprintln(w, BuildResponse(privateKey, 0, "error", ResponseData{}))
}
